@@ -10,6 +10,7 @@ import sys
 # Operating systems.
 WINDOWS = sys.platform == "win32"
 LINUX = sys.platform.startswith("linux")
+OSX = sys.platform == "darwin"
 
 # Python implementations.
 CPYTHON = (platform.python_implementation() == "CPython")
@@ -50,21 +51,12 @@ class PYBEHAVIOR:
     if pep626:
         optimize_if_not_debug2 = False
 
+    # Yet another way to optimize "if not __debug__"?
+    optimize_if_not_debug3 = (PYPY and PYVERSION >= (3, 8))
+
     # Can co_lnotab have negative deltas?
     negative_lnotab = not (PYPY and PYPYVERSION < (7, 2))
 
-    # Do .pyc files conform to PEP 552? Hash-based pyc's.
-    hashed_pyc_pep552 = (PYVERSION >= (3, 7, 0, 'alpha', 4))
-
-    # Python 3.7.0b3 changed the behavior of the sys.path[0] entry for -m. It
-    # used to be an empty string (meaning the current directory). It changed
-    # to be the actual path to the current directory, so that os.chdir wouldn't
-    # affect the outcome.
-    actual_syspath0_dash_m = (
-        (CPYTHON and (PYVERSION >= (3, 7, 0, 'beta', 3))) or
-        (PYPY and (PYPYVERSION >= (7, 3, 4)))
-        )
-
     # 3.7 changed how functions with only docstrings are numbered.
     docstring_only_function = (not PYPY) and ((3, 7, 0, 'beta', 5) <= PYVERSION <= (3, 10))
 
@@ -77,7 +69,7 @@ class PYBEHAVIOR:
     # When a function is decorated, does the trace function get called for the
     # @-line and also the def-line (new behavior in 3.8)? Or just the @-line
     # (old behavior)?
-    trace_decorated_def = (PYVERSION >= (3, 8))
+    trace_decorated_def = (CPYTHON and PYVERSION >= (3, 8))
 
     # Are while-true loops optimized into absolute jumps with no loop setup?
     nix_while_true = (PYVERSION >= (3, 8))
@@ -108,6 +100,10 @@ class PYBEHAVIOR:
     # Some words are keywords in some places, identifiers in other places.
     soft_keywords = (PYVERSION >= (3, 10))
 
+    # CPython 3.11 now jumps to the decorator line again while executing
+    # the decorator.
+    trace_decorator_line_again = (PYVERSION > (3, 11, 0, 'alpha', 3, 0))
+
 
 # Coverage.py specifics.
 
@@ -121,3 +117,12 @@ METACOV = os.getenv('COVERAGE_COVERAGE', '') != ''
 # Even when running tests, you can use COVERAGE_TESTING=0 to disable the
 # test-specific behavior like contracts.
 TESTING = os.getenv('COVERAGE_TESTING', '') == 'True'
+
+# Environment COVERAGE_NO_CONTRACTS=1 can turn off contracts while debugging
+# tests to remove noise from stack traces.
+# $set_env.py: COVERAGE_NO_CONTRACTS - Disable PyContracts to simplify stack traces.
+USE_CONTRACTS = (
+    TESTING
+    and not bool(int(os.environ.get("COVERAGE_NO_CONTRACTS", 0)))
+    and (PYVERSION < (3, 11))
+)
