@@ -183,10 +183,9 @@ class WithTest(CoverageTest):
     """Arc-measuring tests involving context managers."""
 
     def test_with(self):
+        arcz = ".1 .2 23 34 4. 16 6."
         if env.PYBEHAVIOR.exit_through_with:
-            arcz = ".1 .2 23 34 42 2. 16 6."
-        else:
-            arcz = ".1 .2 23 34 4. 16 6."
+            arcz = arcz.replace("4.", "42 2.")
         self.check_coverage("""\
             def example():
                 with open("test", "w") as f:
@@ -199,10 +198,9 @@ class WithTest(CoverageTest):
             )
 
     def test_with_return(self):
+        arcz = ".1 .2 23 34 4. 16 6."
         if env.PYBEHAVIOR.exit_through_with:
-            arcz = ".1 .2 23 34 42 2. 16 6."
-        else:
-            arcz = ".1 .2 23 34 4. 16 6."
+            arcz = arcz.replace("4.", "42 2.")
         self.check_coverage("""\
             def example():
                 with open("test", "w") as f:
@@ -216,10 +214,9 @@ class WithTest(CoverageTest):
 
     def test_bug_146(self):
         # https://github.com/nedbat/coveragepy/issues/146
+        arcz = ".1 12 23 34 41 15 5."
         if env.PYBEHAVIOR.exit_through_with:
-            arcz = ".1 12 23 32 24 41 15 5."
-        else:
-            arcz = ".1 12 23 34 41 15 5."
+            arcz = arcz.replace("34", "32 24")
         self.check_coverage("""\
             for i in range(2):
                 with open("test", "w") as f:
@@ -231,10 +228,9 @@ class WithTest(CoverageTest):
             )
 
     def test_nested_with_return(self):
+        arcz = ".1 .2 23 34 45 56 6. 18 8."
         if env.PYBEHAVIOR.exit_through_with:
-            arcz = ".1 .2 23 34 45 56 64 42 2. 18 8."
-        else:
-            arcz = ".1 .2 23 34 45 56 6. 18 8."
+            arcz = arcz.replace("6.", "64 42 2.")
         self.check_coverage("""\
             def example(x):
                 with open("test", "w") as f2:
@@ -249,10 +245,9 @@ class WithTest(CoverageTest):
             )
 
     def test_break_through_with(self):
+        arcz = ".1 12 23 34 45 15 5."
         if env.PYBEHAVIOR.exit_through_with:
-            arcz = ".1 12 23 34 42 25 15 5."
-        else:
-            arcz = ".1 12 23 34 45 15 5."
+            arcz = arcz.replace("45", "42 25")
         self.check_coverage("""\
             for i in range(1+1):
                 with open("test", "w") as f:
@@ -265,10 +260,9 @@ class WithTest(CoverageTest):
             )
 
     def test_continue_through_with(self):
+        arcz = ".1 12 23 34 41 15 5."
         if env.PYBEHAVIOR.exit_through_with:
-            arcz = ".1 12 23 34 42 21 15 5."
-        else:
-            arcz = ".1 12 23 34 41 15 5."
+            arcz = arcz.replace("41", "42 21")
         self.check_coverage("""\
             for i in range(1+1):
                 with open("test", "w") as f:
@@ -279,7 +273,10 @@ class WithTest(CoverageTest):
             arcz=arcz,
             )
 
-    @pytest.mark.skipif(env.PYVERSION[:2] >= (3, 11), reason="avoid a 3.11 bug: 45709")
+    @pytest.mark.xfail(
+        (3, 11) <= env.PYVERSION <= (3, 11, 0, 'alpha', 2, 0),
+        reason="avoid a 3.11 bug: 45709"
+    )
     # https://github.com/nedbat/coveragepy/issues/1270
     def test_raise_through_with(self):
         if env.PYBEHAVIOR.exit_through_with:
@@ -311,9 +308,7 @@ class WithTest(CoverageTest):
 
     def test_untaken_raise_through_with(self):
         if env.PYBEHAVIOR.exit_through_with:
-            #arcz = ".1 12 28 89 9. AB B.  -23 3-2 34 45 56 53 63 37 7-2"
             arcz = ".1 12 28 89 9. AB B.  -23 34 45 56 53 63 37 7-2"
-            #arcz_missing = "3-2 56 63 AB B."
             arcz_missing = "56 63 AB B."
         else:
             arcz = ".1 12 28 89 9. AB B.  -23 34 45 56 6-2 57 7-2"
@@ -577,7 +572,7 @@ class LoopArcTest(CoverageTest):
         )
 
     # https://bugs.python.org/issue44672
-    @pytest.mark.skipif(env.PYVERSION < (3, 10), reason="<3.10 traced final pass incorrectly")
+    @pytest.mark.xfail(env.PYVERSION < (3, 10), reason="<3.10 traced final pass incorrectly")
     def test_incorrect_loop_exit_bug_1175(self):
         self.check_coverage("""\
             def wrong_loop(x):
@@ -594,7 +589,7 @@ class LoopArcTest(CoverageTest):
         )
 
     # https://bugs.python.org/issue44672
-    @pytest.mark.skipif(env.PYVERSION < (3, 10), reason="<3.10 traced final pass incorrectly")
+    @pytest.mark.xfail(env.PYVERSION < (3, 10), reason="<3.10 traced final pass incorrectly")
     def test_incorrect_if_bug_1175(self):
         self.check_coverage("""\
             def wrong_loop(x):
@@ -853,10 +848,9 @@ class ExceptionArcTest(CoverageTest):
 
 
     def test_break_through_finally(self):
+        arcz = ".1 12 23 34 3D 45 56 67 68 7A    AD 8A A3 BC CD D."
         if env.PYBEHAVIOR.finally_jumps_back:
-            arcz = ".1 12 23 34 3D 45 56 67 68 7A 7D 8A A3 A7 BC CD D."
-        else:
-            arcz = ".1 12 23 34 3D 45 56 67 68 7A 8A A3 AD BC CD D."
+            arcz = arcz.replace("AD", "A7 7D")
         self.check_coverage("""\
             a, c, d, i = 1, 1, 1, 99
             try:
@@ -897,10 +891,9 @@ class ExceptionArcTest(CoverageTest):
         )
 
     def test_continue_through_finally(self):
+        arcz = ".1 12 23 34 3D 45 56 67 68       7A 8A A3 BC CD D."
         if env.PYBEHAVIOR.finally_jumps_back:
-            arcz = ".1 12 23 34 3D 45 56 67 68 73 7A 8A A3 A7 BC CD D."
-        else:
-            arcz = ".1 12 23 34 3D 45 56 67 68 7A 8A A3 BC CD D."
+            arcz += " 73 A7"
         self.check_coverage("""\
             a, b, c, d, i = 1, 1, 1, 1, 99
             try:
@@ -1056,10 +1049,9 @@ class ExceptionArcTest(CoverageTest):
         )
 
     def test_return_finally(self):
+        arcz = ".1 12 29 9A AB BC C-1   -23 34 45  7-2   57 38 8-2"
         if env.PYBEHAVIOR.finally_jumps_back:
-            arcz = ".1 12 29 9A AB BC C-1   -23 34 45 5-2 57 75 38 8-2"
-        else:
-            arcz = ".1 12 29 9A AB BC C-1   -23 34 45 57 7-2 38 8-2"
+            arcz = arcz.replace("7-2", "75 5-2")
         self.check_coverage("""\
             a = [1]
             def check_token(data):
@@ -1078,22 +1070,15 @@ class ExceptionArcTest(CoverageTest):
         )
 
     def test_except_jump_finally(self):
+        arcz = (
+            ".1 1Q QR RS ST TU U. " +
+            ".2 23 34 45 56 4O 6L " +
+            "78 89 9A AL   8B BC CD DL   BE EF FG GL   EH HI IJ JL  HL " +
+            "LO L4 L. LM " +
+            "MN NO O."
+        )
         if env.PYBEHAVIOR.finally_jumps_back:
-            arcz = (
-                ".1 1Q QR RS ST TU U. " +
-                ".2 23 34 45 56 4O 6L " +
-                "78 89 9A AL LA AO  8B BC CD DL LD D4  BE EF FG GL LG G.  EH HI IJ JL  HL " +
-                "L4 LM " +
-                "MN NO O."
-            )
-        else:
-            arcz = (
-                ".1 1Q QR RS ST TU U. " +
-                ".2 23 34 45 56 4O 6L " +
-                "78 89 9A AL  8B BC CD DL  BE EF FG GL  EH HI IJ JL  HL " +
-                "LO L4 L. LM " +
-                "MN NO O."
-            )
+            arcz = arcz.replace("LO", "LA AO").replace("L4", "L4 LD D4").replace("L.", "LG G.")
         self.check_coverage("""\
             def func(x):
                 a = f = g = 2
@@ -1132,22 +1117,15 @@ class ExceptionArcTest(CoverageTest):
         )
 
     def test_else_jump_finally(self):
+        arcz = (
+            ".1 1S ST TU UV VW W. " +
+            ".2 23 34 45 56 6A 78 8N 4Q " +
+            "AB BC CN  AD DE EF FN  DG GH HI IN  GJ JK KL LN  JN " +
+            "N4 NQ N. NO " +
+            "OP PQ Q."
+        )
         if env.PYBEHAVIOR.finally_jumps_back:
-            arcz = (
-                ".1 1S ST TU UV VW W. " +
-                ".2 23 34 45 56 6A 78 8N 4Q " +
-                "AB BC CN NC CQ  AD DE EF FN NF F4  DG GH HI IN NI I.  GJ JK KL LN  JN " +
-                "N4 NO " +
-                "OP PQ Q."
-            )
-        else:
-            arcz = (
-                ".1 1S ST TU UV VW W. " +
-                ".2 23 34 45 56 6A 78 8N 4Q " +
-                "AB BC CN  AD DE EF FN  DG GH HI IN  GJ JK KL LN  JN " +
-                "N4 NQ N. NO " +
-                "OP PQ Q."
-            )
+            arcz = arcz.replace("NQ", "NC CQ").replace("N4", "N4 NF F4").replace("N.", "NI I.")
         self.check_coverage("""\
             def func(x):
                 a = f = g = 2
@@ -1217,6 +1195,11 @@ class YieldTest(CoverageTest):
             arcz=".1 19 9.  .2 23 34 45 56 63 37 7.",
         )
 
+    # https://bugs.python.org/issue46225
+    @pytest.mark.xfail(
+        env.PYVERSION[:5] == (3, 11, 0, 'alpha', 3),
+        reason="avoid 3.11 bug: bpo46225",
+    )
     def test_bug_308(self):
         self.check_coverage("""\
             def run():
@@ -1251,6 +1234,10 @@ class YieldTest(CoverageTest):
             arcz=".1 14 45 54 4.  .2 2.  -22 2-2",
         )
 
+    @pytest.mark.xfail(
+        ((3, 11, 0, "alpha", 4, 0) == env.PYVERSION) and not env.C_TRACER,
+        reason="avoid a 3.11 bug: https://bugs.python.org/issue46389",
+    )
     def test_bug_324(self):
         # This code is tricky: the list() call pulls all the values from gen(),
         # but each of them is a generator itself that is never iterated.  As a
@@ -1301,7 +1288,6 @@ class YieldTest(CoverageTest):
             list(gen([1,2,3]))
             """,
             arcz=".1 19 9.  .2 23 34 45 56 63 37 7.",
-            arcz_unpredicted="5.",
         )
 
     def test_abandoned_yield(self):
@@ -1321,8 +1307,8 @@ class YieldTest(CoverageTest):
         )
 
 
+@pytest.mark.xfail(env.PYVERSION <= (3, 10, 0, 'beta', 4, 0), reason="3.10.0b4 had bugs")
 @pytest.mark.skipif(not env.PYBEHAVIOR.match_case, reason="Match-case is new in 3.10")
-@pytest.mark.skipif(env.PYVERSION <= (3, 10, 0, 'beta', 4, 0), reason="3.10.0b4 had bugs")
 class MatchCaseTest(CoverageTest):
     """Tests of match-case."""
     def test_match_case_with_default(self):
@@ -1637,6 +1623,10 @@ class MiscArcTest(CoverageTest):
         self.check_coverage(code, arcs=[(-1, 1), (1, 2*n+4), (2*n+4, -1)])
         assert self.stdout() == f"{n}\n"
 
+    @pytest.mark.xfail(
+        ((3, 11, 0, "alpha", 4, 0) == env.PYVERSION) and not env.C_TRACER,
+        reason="avoid a 3.11 bug: https://bugs.python.org/issue46389",
+    )
     def test_partial_generators(self):
         # https://github.com/nedbat/coveragepy/issues/475
         # Line 2 is executed completely.
@@ -1664,6 +1654,13 @@ class DecoratorArcTest(CoverageTest):
     """Tests of arcs with decorators."""
 
     def test_function_decorator(self):
+        arcz = (
+            ".1 16 67 7A AE EF F. "     # main line
+            ".2 24 4.   -23 3-2 "       # decorators
+            "-6D D-6 "                  # my_function
+        )
+        if env.PYBEHAVIOR.trace_decorator_line_again:
+            arcz += "A7 76 6A "
         self.check_coverage("""\
             def decorator(arg):
                 def _dec(f):
@@ -1681,13 +1678,17 @@ class DecoratorArcTest(CoverageTest):
             a = 14
             my_function()
             """,
-            arcz=
-                ".1 16 67 7A AE EF F. "     # main line
-                ".2 24 4.   -23 3-2 "       # decorators
-                "-6D D-6 ",                 # my_function
+            arcz=arcz,
         )
 
     def test_class_decorator(self):
+        arcz = (
+            ".1 16 67 6D 7A AE E. "     # main line
+            ".2 24 4.   -23 3-2 "       # decorators
+            "-66 D-6 "                  # MyObject
+        )
+        if env.PYBEHAVIOR.trace_decorator_line_again:
+            arcz += "A7 76 6A "
         self.check_coverage("""\
             def decorator(arg):
                 def _dec(c):
@@ -1704,19 +1705,17 @@ class DecoratorArcTest(CoverageTest):
                 X = 13
             a = 14
             """,
-            arcz=
-                ".1 16 67 6D 7A AE E. "     # main line
-                ".2 24 4.   -23 3-2 "       # decorators
-                "-66 D-6 ",                 # MyObject
+            arcz=arcz,
         )
 
-    def test_bug_466(self):
+    def test_bug_466a(self):
         # A bad interaction between decorators and multi-line list assignments,
         # believe it or not...!
+        arcz = ".1 1A A.  13 3.     -35 58 8-3 "
         if env.PYBEHAVIOR.trace_decorated_def:
-            arcz = ".1 1A A.  13 34 4.  -35 58 8-3"
-        else:
-            arcz = ".1 1A A.  13 3.     -35 58 8-3"
+            arcz = arcz.replace("3.", "34 4.")
+        if env.PYBEHAVIOR.trace_decorator_line_again:
+            arcz += "43 "
         self.check_coverage("""\
             class Parser(object):
 
@@ -1731,10 +1730,15 @@ class DecoratorArcTest(CoverageTest):
             """,
             arcz=arcz,
         )
+
+    def test_bug_466b(self):
+        # A bad interaction between decorators and multi-line list assignments,
+        # believe it or not...!
+        arcz = ".1 1A A.  13 3.     -35 58 8-3 "
         if env.PYBEHAVIOR.trace_decorated_def:
-            arcz = ".1 1A A.  13 34 4.  -35 58 8-3"
-        else:
-            arcz = ".1 1A A.  13 3.     -35 58 8-3"
+            arcz = arcz.replace("3.", "34 4.")
+        if env.PYBEHAVIOR.trace_decorator_line_again:
+            arcz += "43 "
         self.check_coverage("""\
             class Parser(object):
 
@@ -1827,15 +1831,23 @@ class LambdaArcTest(CoverageTest):
         )
 
 
+xfail_eventlet_670 = pytest.mark.xfail(
+    env.PYVERSION[:2] == (3, 9) and env.OSX,
+    reason="Avoid an eventlet bug on Mac 3.9: eventlet#670",
+    # https://github.com/eventlet/eventlet/issues/670
+)
+
+
 class AsyncTest(CoverageTest):
     """Tests of the new async and await keywords in Python 3.5"""
 
+    @xfail_eventlet_670
     def test_async(self):
         self.check_coverage("""\
             import asyncio
 
             async def compute(x, y):                        # 3
-                print("Compute %s + %s ..." % (x, y))
+                print(f"Compute {x} + {y} ...")
                 await asyncio.sleep(0.001)
                 return x + y                                # 6
 
@@ -1843,7 +1855,7 @@ class AsyncTest(CoverageTest):
                 result = (0 +
                             await compute(x, y)             # A
                 )
-                print("%s + %s = %s" % (x, y, result))
+                print(f"{x} + {y} = {result}")
 
             loop = asyncio.new_event_loop()                 # E
             loop.run_until_complete(print_sum(1, 2))
@@ -1853,10 +1865,10 @@ class AsyncTest(CoverageTest):
                 ".1 13 38 8E EF FG G. " +
                 "-34 45 56 6-3 " +
                 "-89 9C C-8",
-            arcz_unpredicted="5-3 9-8",
         )
         assert self.stdout() == "Compute 1 + 2 ...\n1 + 2 = 3\n"
 
+    @xfail_eventlet_670
     def test_async_for(self):
         self.check_coverage("""\
             import asyncio
@@ -1911,10 +1923,11 @@ class AsyncTest(CoverageTest):
         )
 
     def test_async_decorator(self):
+        arcz = ".1 14 4.     .2 2.  -46 6-4 "
         if env.PYBEHAVIOR.trace_decorated_def:
-            arcz = ".1 14 45 5.  .2 2.  -46 6-4"
-        else:
-            arcz = ".1 14 4.     .2 2.  -46 6-4"
+            arcz = arcz.replace("4.", "45 5.")
+        if env.PYBEHAVIOR.trace_decorator_line_again:
+            arcz += "54 "
         self.check_coverage("""\
             def wrap(f):        # 1
                 return f
@@ -1931,7 +1944,7 @@ class AsyncTest(CoverageTest):
     # https://bugs.python.org/issue44621
     @pytest.mark.skipif(env.PYVERSION[:2] == (3, 9), reason="avoid a 3.9 bug: 44621")
     @pytest.mark.skipif(env.PYVERSION < (3, 7), reason="need asyncio.run")
-    def test_bug1158(self):
+    def test_bug_1158(self):
         self.check_coverage("""\
             import asyncio
 
@@ -1955,12 +1968,13 @@ class AsyncTest(CoverageTest):
 
     # https://github.com/nedbat/coveragepy/issues/1176
     # https://bugs.python.org/issue44622
-    @pytest.mark.skipif(
-        (3, 10, 0, "alpha", 0, 0) <= env.PYVERSION[:6] <= (3, 10, 0, "beta", 4, 0),
+    @pytest.mark.xfail(
+        (3, 10, 0, "alpha", 0, 0) <= env.PYVERSION <= (3, 10, 0, "beta", 4, 0),
         reason="avoid a 3.10 bug fixed after beta 4: 44622"
     )
+    @xfail_eventlet_670
     @pytest.mark.skipif(env.PYVERSION < (3, 7), reason="need asyncio.run")
-    def test_bug1176(self):
+    def test_bug_1176(self):
         self.check_coverage("""\
             import asyncio
 
@@ -1979,11 +1993,11 @@ class AsyncTest(CoverageTest):
 
     # https://github.com/nedbat/coveragepy/issues/1205
     # https://bugs.python.org/issue44840
-    @pytest.mark.skipif(
-        (3, 10, 0, "alpha", 0, 0) <= env.PYVERSION[:6] <= (3, 10, 0, "candidate", 1, 0),
+    @pytest.mark.xfail(
+        (3, 10, 0, "alpha", 0, 0) <= env.PYVERSION <= (3, 10, 0, "candidate", 1, 0),
         reason="avoid a 3.10 bug fixed after rc1: 44840"
     )
-    def test_bug1205(self):
+    def test_bug_1205(self):
         self.check_coverage("""\
             def func():
                 if T(2):
