@@ -4,6 +4,7 @@
 """Test cmdline.py for coverage.py."""
 
 import pprint
+import re
 import sys
 import textwrap
 
@@ -13,9 +14,9 @@ import pytest
 import coverage
 import coverage.cmdline
 from coverage import env
+from coverage.control import DEFAULT_DATAFILE
 from coverage.config import CoverageConfig
-from coverage.data import CoverageData
-from coverage.exceptions import ExceptionDuringRun
+from coverage.exceptions import _ExceptionDuringRun
 from coverage.version import __url__
 
 from tests.coveragetest import CoverageTest, OK, ERR, command_line
@@ -52,7 +53,12 @@ class BaseCmdLineTest(CoverageTest):
         ignore_errors=None, include=None, omit=None, morfs=[], outfile=None,
         contexts=None, pretty_print=None, show_contexts=None,
     )
+    _defaults.Coverage().lcov_report(
+        ignore_errors=None, include=None, omit=None, morfs=[], outfile=None,
+        contexts=None,
+    )
     _defaults.Coverage(
+        data_file=DEFAULT_DATAFILE,
         cover_pylib=None, data_suffix=None, timid=None, branch=None,
         config_file=True, source=None, include=None, omit=None, debug=None,
         concurrency=None, check_preimported=True, context=None, messages=True,
@@ -76,6 +82,7 @@ class BaseCmdLineTest(CoverageTest):
         cov.html_report.return_value = 50.0
         cov.xml_report.return_value = 50.0
         cov.json_report.return_value = 50.0
+        cov.lcov_report.return_value = 50.0
 
         return mk
 
@@ -245,6 +252,11 @@ class CmdLineTest(BaseCmdLineTest):
             cov.combine(None, strict=True, keep=False)
             cov.save()
             """)
+        self.cmd_executes("combine --data-file=foo.cov", """\
+            cov = Coverage(data_file="foo.cov")
+            cov.combine(None, strict=True, keep=False)
+            cov.save()
+            """)
 
     def test_combine_doesnt_confuse_options_with_args(self):
         # https://github.com/nedbat/coveragepy/issues/385
@@ -259,9 +271,13 @@ class CmdLineTest(BaseCmdLineTest):
             cov.save()
             """)
 
-    def test_debug(self):
-        self.cmd_help("debug", "What information would you like: config, data, sys, premain?")
-        self.cmd_help("debug foo", "Don't know what you mean by 'foo'")
+    @pytest.mark.parametrize("cmd, output", [
+        ("debug", "What information would you like: config, data, sys, premain?"),
+        ("debug foo", "Don't know what you mean by 'foo'"),
+        ("debug sys config", "Only one topic at a time, please"),
+    ])
+    def test_debug(self, cmd, output):
+        self.cmd_help(cmd, output)
 
     def test_debug_sys(self):
         self.command_line("debug sys")
@@ -276,12 +292,30 @@ class CmdLineTest(BaseCmdLineTest):
         assert "skip_covered:" in out
         assert "skip_empty:" in out
 
+    def test_debug_premain(self):
+        self.command_line("debug premain")
+        out = self.stdout()
+        #   ... many lines ...
+        #   pytest_pyfunc_call : /Users/ned/cov/trunk/.tox/py39/site-packages/_pytest/python.py:183
+        #   test_debug_premain : /Users/ned/cov/trunk/tests/test_cmdline.py:284
+        #         command_line : /Users/ned/cov/trunk/tests/coveragetest.py:309
+        #         command_line : /Users/ned/cov/trunk/tests/coveragetest.py:472
+        #         command_line : /Users/ned/cov/trunk/coverage/cmdline.py:592
+        #             do_debug : /Users/ned/cov/trunk/coverage/cmdline.py:804
+        assert re.search(r"(?m)^\s+test_debug_premain : .*[/\\]tests[/\\]test_cmdline.py:\d+$", out)
+        assert re.search(r"(?m)^\s+command_line : .*[/\\]coverage[/\\]cmdline.py:\d+$", out)
+        assert re.search(r"(?m)^\s+do_debug : .*[/\\]coverage[/\\]cmdline.py:\d+$", out)
+
     def test_erase(self):
         # coverage erase
         self.cmd_executes("erase", """\
             cov = Coverage()
             cov.erase()
             """)
+        self.cmd_executes("erase --data-file=foo.cov", """\
+            cov = Coverage(data_file="foo.cov")
+            cov.erase()
+            """)
 
     def test_version(self):
         # coverage --version
@@ -420,6 +454,49 @@ class CmdLineTest(BaseCmdLineTest):
             cov.json_report()
             """)
 
+    def test_lcov(self):
+        # coverage lcov [-i] [--omit DIR,...] [FILE1 FILE2 ...]
+        self.cmd_executes("lcov", """\
+            cov = Coverage()
+            cov.load()
+            cov.lcov_report()
+            """)
+        self.cmd_executes("lcov -i", """\
+            cov = Coverage()
+            cov.load()
+            cov.lcov_report(ignore_errors=True)
+            """)
+        self.cmd_executes("lcov -o mylcov.foo", """\
+            cov = Coverage()
+            cov.load()
+            cov.lcov_report(outfile="mylcov.foo")
+            """)
+        self.cmd_executes("lcov -o -", """\
+            cov = Coverage()
+            cov.load()
+            cov.lcov_report(outfile="-")
+            """)
+        self.cmd_executes("lcov --omit fooey", """\
+            cov = Coverage(omit=["fooey"])
+            cov.load()
+            cov.lcov_report(omit=["fooey"])
+            """)
+        self.cmd_executes("lcov --omit fooey,booey", """\
+            cov = Coverage(omit=["fooey", "booey"])
+            cov.load()
+            cov.lcov_report(omit=["fooey", "booey"])
+            """)
+        self.cmd_executes("lcov -q", """\
+            cov = Coverage(messages=False)
+            cov.load()
+            cov.lcov_report()
+            """)
+        self.cmd_executes("lcov --quiet", """\
+            cov = Coverage(messages=False)
+            cov.load()
+            cov.lcov_report()
+            """)
+
     def test_report(self):
         # coverage report [-m] [-i] [-o DIR,...] [FILE1 FILE2 ...]
         self.cmd_executes("report", """\
@@ -492,6 +569,11 @@ class CmdLineTest(BaseCmdLineTest):
             cov.load()
             cov.report(sort='-foo')
             """)
+        self.cmd_executes("report --data-file=foo.cov.2", """\
+            cov = Coverage(data_file="foo.cov.2")
+            cov.load()
+            cov.report(show_missing=None)
+            """)
 
     def test_run(self):
         # coverage run [-p] [-L] [--timid] MODULE.py [ARG1 ARG2 ...]
@@ -592,7 +674,7 @@ class CmdLineTest(BaseCmdLineTest):
             cov.save()
             """)
         self.cmd_executes("run --concurrency=gevent foo.py", """\
-            cov = Coverage(concurrency='gevent')
+            cov = Coverage(concurrency=['gevent'])
             runner = PyRunner(['foo.py'], as_module=False)
             runner.prepare()
             cov.start()
@@ -601,7 +683,25 @@ class CmdLineTest(BaseCmdLineTest):
             cov.save()
             """)
         self.cmd_executes("run --concurrency=multiprocessing foo.py", """\
-            cov = Coverage(concurrency='multiprocessing')
+            cov = Coverage(concurrency=['multiprocessing'])
+            runner = PyRunner(['foo.py'], as_module=False)
+            runner.prepare()
+            cov.start()
+            runner.run()
+            cov.stop()
+            cov.save()
+            """)
+        self.cmd_executes("run --concurrency=gevent,thread foo.py", """\
+            cov = Coverage(concurrency=['gevent', 'thread'])
+            runner = PyRunner(['foo.py'], as_module=False)
+            runner.prepare()
+            cov.start()
+            runner.run()
+            cov.stop()
+            cov.save()
+            """)
+        self.cmd_executes("run --data-file=output.coverage foo.py", """\
+            cov = Coverage(data_file="output.coverage")
             runner = PyRunner(['foo.py'], as_module=False)
             runner.prepare()
             cov.start()
@@ -609,19 +709,6 @@ class CmdLineTest(BaseCmdLineTest):
             cov.stop()
             cov.save()
             """)
-
-    def test_bad_concurrency(self):
-        self.command_line("run --concurrency=nothing", ret=ERR)
-        err = self.stderr()
-        assert "option --concurrency: invalid choice: 'nothing'" in err
-
-    def test_no_multiple_concurrency(self):
-        # You can't use multiple concurrency values on the command line.
-        # I would like to have a better message about not allowing multiple
-        # values for this option, but optparse is not that flexible.
-        self.command_line("run --concurrency=multiprocessing,gevent foo.py", ret=ERR)
-        err = self.stderr()
-        assert "option --concurrency: invalid choice: 'multiprocessing,gevent'" in err
 
     def test_multiprocessing_needs_config_file(self):
         # You can't use command-line args to add options to multiprocessing
@@ -808,33 +895,51 @@ class CmdLineWithFilesTest(BaseCmdLineTest):
     run_in_temp_dir = True
 
     def test_debug_data(self):
-        data = CoverageData()
-        data.add_lines({
-            "file1.py": dict.fromkeys(range(1, 18)),
-            "file2.py": dict.fromkeys(range(1, 24)),
-        })
-        data.add_file_tracers({"file1.py": "a_plugin"})
-        data.write()
+        data = self.make_data_file(
+            lines={
+                "file1.py": range(1, 18),
+                "file2.py": range(1, 24),
+            },
+            file_tracers={"file1.py": "a_plugin"},
+        )
 
         self.command_line("debug data")
-        assert self.stdout() == textwrap.dedent("""\
+        assert self.stdout() == textwrap.dedent(f"""\
             -- data ------------------------------------------------------
-            path: FILENAME
+            path: {data.data_filename()}
             has_arcs: False
-
             2 files:
             file1.py: 17 lines [a_plugin]
             file2.py: 23 lines
-            """).replace("FILENAME", data.data_filename())
+            """)
 
-    def test_debug_data_with_no_data(self):
-        data = CoverageData()
+    def test_debug_data_with_no_data_file(self):
+        data = self.make_data_file()
         self.command_line("debug data")
-        assert self.stdout() == textwrap.dedent("""\
+        assert self.stdout() == textwrap.dedent(f"""\
             -- data ------------------------------------------------------
-            path: FILENAME
-            No data collected
-            """).replace("FILENAME", data.data_filename())
+            path: {data.data_filename()}
+            No data collected: file doesn't exist
+            """)
+
+    def test_debug_combinable_data(self):
+        data1 = self.make_data_file(lines={"file1.py": range(1, 18), "file2.py": [1]})
+        data2 = self.make_data_file(suffix="123", lines={"file2.py": range(1, 10)})
+
+        self.command_line("debug data")
+        assert self.stdout() == textwrap.dedent(f"""\
+            -- data ------------------------------------------------------
+            path: {data1.data_filename()}
+            has_arcs: False
+            2 files:
+            file1.py: 17 lines
+            file2.py: 1 line
+            -----
+            path: {data2.data_filename()}
+            has_arcs: False
+            1 file:
+            file2.py: 9 lines
+            """)
 
 
 class CmdLineStdoutTest(BaseCmdLineTest):
@@ -931,7 +1036,7 @@ class CmdMainTest(CoverageTest):
                 try:
                     raise Exception("oh noes!")
                 except:
-                    raise ExceptionDuringRun(*sys.exc_info())
+                    raise _ExceptionDuringRun(*sys.exc_info())
             elif argv[0] == 'internalraise':
                 raise ValueError("coverage is broken")
             elif argv[0] == 'exit':
@@ -974,12 +1079,13 @@ class CmdMainTest(CoverageTest):
 class CoverageReportingFake:
     """A fake Coverage.coverage test double."""
     # pylint: disable=missing-function-docstring
-    def __init__(self, report_result, html_result, xml_result, json_report):
+    def __init__(self, report_result, html_result, xml_result, json_report, lcov_result):
         self.config = CoverageConfig()
         self.report_result = report_result
         self.html_result = html_result
         self.xml_result = xml_result
         self.json_result = json_report
+        self.lcov_result = lcov_result
 
     def set_option(self, optname, optvalue):
         self.config.set_option(optname, optvalue)
@@ -1002,28 +1108,34 @@ class CoverageReportingFake:
     def json_report(self, *args_unused, **kwargs_unused):
         return self.json_result
 
+    def lcov_report(self, *args_unused, **kwargs_unused):
+        return self.lcov_result
 
 @pytest.mark.parametrize("results, fail_under, cmd, ret", [
     # Command-line switch properly checks the result of reporting functions.
-    ((20, 30, 40, 50), None, "report --fail-under=19", 0),
-    ((20, 30, 40, 50), None, "report --fail-under=21", 2),
-    ((20, 30, 40, 50), None, "html --fail-under=29", 0),
-    ((20, 30, 40, 50), None, "html --fail-under=31", 2),
-    ((20, 30, 40, 50), None, "xml --fail-under=39", 0),
-    ((20, 30, 40, 50), None, "xml --fail-under=41", 2),
-    ((20, 30, 40, 50), None, "json --fail-under=49", 0),
-    ((20, 30, 40, 50), None, "json --fail-under=51", 2),
+    ((20, 30, 40, 50, 60), None, "report --fail-under=19", 0),
+    ((20, 30, 40, 50, 60), None, "report --fail-under=21", 2),
+    ((20, 30, 40, 50, 60), None, "html --fail-under=29", 0),
+    ((20, 30, 40, 50, 60), None, "html --fail-under=31", 2),
+    ((20, 30, 40, 50, 60), None, "xml --fail-under=39", 0),
+    ((20, 30, 40, 50, 60), None, "xml --fail-under=41", 2),
+    ((20, 30, 40, 50, 60), None, "json --fail-under=49", 0),
+    ((20, 30, 40, 50, 60), None, "json --fail-under=51", 2),
+    ((20, 30, 40, 50, 60), None, "lcov --fail-under=59", 0),
+    ((20, 30, 40, 50, 60), None, "lcov --fail-under=61", 2),
     # Configuration file setting properly checks the result of reporting.
-    ((20, 30, 40, 50), 19, "report", 0),
-    ((20, 30, 40, 50), 21, "report", 2),
-    ((20, 30, 40, 50), 29, "html", 0),
-    ((20, 30, 40, 50), 31, "html", 2),
-    ((20, 30, 40, 50), 39, "xml", 0),
-    ((20, 30, 40, 50), 41, "xml", 2),
-    ((20, 30, 40, 50), 49, "json", 0),
-    ((20, 30, 40, 50), 51, "json", 2),
+    ((20, 30, 40, 50, 60), 19, "report", 0),
+    ((20, 30, 40, 50, 60), 21, "report", 2),
+    ((20, 30, 40, 50, 60), 29, "html", 0),
+    ((20, 30, 40, 50, 60), 31, "html", 2),
+    ((20, 30, 40, 50, 60), 39, "xml", 0),
+    ((20, 30, 40, 50, 60), 41, "xml", 2),
+    ((20, 30, 40, 50, 60), 49, "json", 0),
+    ((20, 30, 40, 50, 60), 51, "json", 2),
+    ((20, 30, 40, 50, 60), 59, "lcov", 0),
+    ((20, 30, 40, 50, 60), 61, "lcov", 2),
     # Command-line overrides configuration.
-    ((20, 30, 40, 50), 19, "report --fail-under=21", 2),
+    ((20, 30, 40, 50, 60), 19, "report --fail-under=21", 2),
 ])
 def test_fail_under(results, fail_under, cmd, ret):
     cov = CoverageReportingFake(*results)
