@@ -17,11 +17,11 @@ import pytest
 import coverage
 from coverage import env
 from coverage.data import line_counts
+from coverage.exceptions import ConfigError
 from coverage.files import abs_file
 from coverage.misc import import_local_file
 
 from tests.coveragetest import CoverageTest
-from tests.helpers import remove_files
 
 
 # These libraries aren't always available, we'll skip tests if they aren't.
@@ -193,7 +193,7 @@ def cant_trace_msg(concurrency, the_module):
         expected_out = None
     else:
         expected_out = (
-            f"Can't support concurrency={concurrency} with PyTracer, only threads are supported\n"
+            f"Can't support concurrency={concurrency} with PyTracer, only threads are supported.\n"
         )
     return expected_out
 
@@ -212,16 +212,16 @@ class ConcurrencyTest(CoverageTest):
         is the text we expect the code to produce.
 
         """
-
         self.make_file("try_it.py", code)
 
-        cmd = "coverage run --concurrency=%s try_it.py" % concurrency
+        cmd = f"coverage run --concurrency={concurrency} try_it.py"
         out = self.run_command(cmd)
 
         expected_cant_trace = cant_trace_msg(concurrency, the_module)
 
         if expected_cant_trace is not None:
             assert out == expected_cant_trace
+            pytest.skip(f"Can't test: {expected_cant_trace}")
         else:
             # We can fully measure the code if we are using the C tracer, which
             # can support all the concurrency, or if we are using threads.
@@ -260,6 +260,8 @@ class ConcurrencyTest(CoverageTest):
         code = SIMPLE.format(QLIMIT=self.QLIMIT)
         self.try_some_code(code, "eventlet", eventlet)
 
+    # https://github.com/nedbat/coveragepy/issues/663
+    @pytest.mark.skipif(env.WINDOWS, reason="gevent has problems on Windows: #663")
     def test_gevent(self):
         code = (GEVENT + SUM_RANGE_Q + PRINT_SUM_RANGE).format(QLIMIT=self.QLIMIT)
         self.try_some_code(code, "gevent", gevent)
@@ -308,6 +310,85 @@ class ConcurrencyTest(CoverageTest):
             """
         self.try_some_code(BUG_330, "eventlet", eventlet, "0\n")
 
+    def test_threads_with_gevent(self):
+        self.make_file("both.py", """\
+            import queue
+            import threading
+
+            import gevent
+
+            def work1(q):
+                q.put(1)
+
+            def gwork(q):
+                gevent.spawn(work1, q).join()
+                q.put(None)
+                print("done")
+
+            q = queue.Queue()
+            t = threading.Thread(target=gwork, args=(q,))
+            t.start()
+            t.join()
+
+            answer = q.get()
+            assert answer == 1
+            """)
+        out = self.run_command("coverage run --concurrency=thread,gevent both.py")
+        if gevent is None:
+            assert out == (
+                "Couldn't trace with concurrency=gevent, the module isn't installed.\n"
+            )
+            pytest.skip("Can't run test without gevent installed.")
+        if not env.C_TRACER:
+            assert out == (
+                "Can't support concurrency=gevent with PyTracer, only threads are supported.\n"
+            )
+            pytest.skip("Can't run gevent with PyTracer")
+
+        assert out == "done\n"
+
+        out = self.run_command("coverage report -m")
+        last_line = self.squeezed_lines(out)[-1]
+        assert re.search(r"TOTAL \d+ 0 100%", last_line)
+
+    def test_bad_concurrency(self):
+        self.make_file("prog.py", "a = 1")
+        msg = "Unknown concurrency choices: nothing"
+        with pytest.raises(ConfigError, match=msg):
+            self.command_line("run --concurrency=nothing prog.py")
+
+    def test_bad_concurrency_in_config(self):
+        self.make_file("prog.py", "a = 1")
+        self.make_file(".coveragerc", "[run]\nconcurrency = nothing\n")
+        msg = "Unknown concurrency choices: nothing"
+        with pytest.raises(ConfigError, match=msg):
+            self.command_line("run prog.py")
+
+    def test_no_multiple_light_concurrency(self):
+        self.make_file("prog.py", "a = 1")
+        msg = "Conflicting concurrency settings: eventlet, gevent"
+        with pytest.raises(ConfigError, match=msg):
+            self.command_line("run --concurrency=gevent,eventlet prog.py")
+
+    def test_no_multiple_light_concurrency_in_config(self):
+        self.make_file("prog.py", "a = 1")
+        self.make_file(".coveragerc", "[run]\nconcurrency = gevent, eventlet\n")
+        msg = "Conflicting concurrency settings: eventlet, gevent"
+        with pytest.raises(ConfigError, match=msg):
+            self.command_line("run prog.py")
+
+
+class WithoutConcurrencyModuleTest(CoverageTest):
+    """Tests of what happens if the requested concurrency isn't installed."""
+
+    @pytest.mark.parametrize("module", ["eventlet", "gevent", "greenlet"])
+    def test_missing_module(self, module):
+        self.make_file("prog.py", "a = 1")
+        sys.modules[module] = None
+        msg = f"Couldn't trace with concurrency={module}, the module isn't installed."
+        with pytest.raises(ConfigError, match=msg):
+            self.command_line(f"run --concurrency={module} prog.py")
+
 
 SQUARE_OR_CUBE_WORK = """
     def work(x):
@@ -356,68 +437,95 @@ MULTI_CODE = """
     """
 
 
+@pytest.fixture(params=["fork", "spawn"], name="start_method")
+def start_method_fixture(request):
+    """Parameterized fixture to choose the start_method for multiprocessing."""
+    start_method = request.param
+    if start_method not in multiprocessing.get_all_start_methods():
+        # Windows doesn't support "fork".
+        pytest.skip(f"start_method={start_method} not supported here")
+    return start_method
+
+
 @pytest.mark.skipif(not multiprocessing, reason="No multiprocessing in this Python")
 @flaky(max_runs=30)         # Sometimes a test fails due to inherent randomness. Try more times.
 class MultiprocessingTest(CoverageTest):
     """Test support of the multiprocessing module."""
 
     def try_multiprocessing_code(
-        self, code, expected_out, the_module, nprocs, concurrency="multiprocessing", args=""
+        self,
+        code,
+        expected_out,
+        the_module,
+        nprocs,
+        start_method,
+        concurrency="multiprocessing",
+        args="",
     ):
         """Run code using multiprocessing, it should produce `expected_out`."""
         self.make_file("multi.py", code)
-        self.make_file(".coveragerc", """\
+        self.make_file(".coveragerc", f"""\
             [run]
-            concurrency = %s
+            concurrency = {concurrency}
             source = .
-            """ % concurrency)
+            """)
 
-        for start_method in ["fork", "spawn"]:
-            if start_method and start_method not in multiprocessing.get_all_start_methods():
-                continue
+        cmd = "coverage run {args} multi.py {start_method}".format(
+            args=args, start_method=start_method,
+        )
+        out = self.run_command(cmd)
+        expected_cant_trace = cant_trace_msg(concurrency, the_module)
 
-            remove_files(".coverage", ".coverage.*")
-            cmd = "coverage run {args} multi.py {start_method}".format(
-                args=args, start_method=start_method,
+        if expected_cant_trace is not None:
+            print(out)
+            assert out == expected_cant_trace
+            pytest.skip(f"Can't test: {expected_cant_trace}")
+        else:
+            assert out.rstrip() == expected_out
+            assert len(glob.glob(".coverage.*")) == nprocs + 1
+
+            out = self.run_command("coverage combine")
+            out_lines = out.splitlines()
+            assert len(out_lines) == nprocs + 1
+            assert all(
+                re.fullmatch(r"Combined data file \.coverage\..*\.\d+\.\d+", line)
+                for line in out_lines
             )
-            out = self.run_command(cmd)
-            expected_cant_trace = cant_trace_msg(concurrency, the_module)
+            out = self.run_command("coverage report -m")
 
-            if expected_cant_trace is not None:
-                assert out == expected_cant_trace
-            else:
-                assert out.rstrip() == expected_out
-                assert len(glob.glob(".coverage.*")) == nprocs + 1
+            last_line = self.squeezed_lines(out)[-1]
+            assert re.search(r"TOTAL \d+ 0 100%", last_line)
 
-                out = self.run_command("coverage combine")
-                out_lines = out.splitlines()
-                assert len(out_lines) == nprocs + 1
-                assert all(
-                    re.fullmatch(r"Combined data file \.coverage\..*\.\d+\.\d+", line)
-                    for line in out_lines
-                )
-                out = self.run_command("coverage report -m")
-
-                last_line = self.squeezed_lines(out)[-1]
-                assert re.search(r"TOTAL \d+ 0 100%", last_line)
-
-    def test_multiprocessing_simple(self):
+    def test_multiprocessing_simple(self, start_method):
         nprocs = 3
         upto = 30
         code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
         total = sum(x*x if x%2 else x*x*x for x in range(upto))
         expected_out = f"{nprocs} pids, total = {total}"
-        self.try_multiprocessing_code(code, expected_out, threading, nprocs)
+        self.try_multiprocessing_code(
+            code,
+            expected_out,
+            threading,
+            nprocs,
+            start_method=start_method,
+        )
 
-    def test_multiprocessing_append(self):
+    def test_multiprocessing_append(self, start_method):
         nprocs = 3
         upto = 30
         code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
         total = sum(x*x if x%2 else x*x*x for x in range(upto))
         expected_out = f"{nprocs} pids, total = {total}"
-        self.try_multiprocessing_code(code, expected_out, threading, nprocs, args="--append")
+        self.try_multiprocessing_code(
+            code,
+            expected_out,
+            threading,
+            nprocs,
+            args="--append",
+            start_method=start_method,
+        )
 
-    def test_multiprocessing_and_gevent(self):
+    def test_multiprocessing_and_gevent(self, start_method):
         nprocs = 3
         upto = 30
         code = (
@@ -426,10 +534,15 @@ class MultiprocessingTest(CoverageTest):
         total = sum(sum(range((x + 1) * 100)) for x in range(upto))
         expected_out = f"{nprocs} pids, total = {total}"
         self.try_multiprocessing_code(
-            code, expected_out, eventlet, nprocs, concurrency="multiprocessing,eventlet"
+            code,
+            expected_out,
+            eventlet,
+            nprocs,
+            concurrency="multiprocessing,eventlet",
+            start_method=start_method,
         )
 
-    def test_multiprocessing_with_branching(self):
+    def test_multiprocessing_with_branching(self, start_method):
         nprocs = 3
         upto = 30
         code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
@@ -443,19 +556,15 @@ class MultiprocessingTest(CoverageTest):
             omit = */site-packages/*
             """)
 
-        for start_method in ["fork", "spawn"]:
-            if start_method and start_method not in multiprocessing.get_all_start_methods():
-                continue
+        out = self.run_command(f"coverage run --rcfile=multi.rc multi.py {start_method}")
+        assert out.rstrip() == expected_out
 
-            out = self.run_command(f"coverage run --rcfile=multi.rc multi.py {start_method}")
-            assert out.rstrip() == expected_out
+        out = self.run_command("coverage combine -q")   # sneak in a test of -q
+        assert out == ""
+        out = self.run_command("coverage report -m")
 
-            out = self.run_command("coverage combine -q")   # sneak in a test of -q
-            assert out == ""
-            out = self.run_command("coverage report -m")
-
-            last_line = self.squeezed_lines(out)[-1]
-            assert re.search(r"TOTAL \d+ 0 \d+ 0 100%", last_line)
+        last_line = self.squeezed_lines(out)[-1]
+        assert re.search(r"TOTAL \d+ 0 \d+ 0 100%", last_line)
 
     def test_multiprocessing_bootstrap_error_handling(self):
         # An exception during bootstrapping will be reported.
@@ -474,7 +583,7 @@ class MultiprocessingTest(CoverageTest):
         assert "Exception during multiprocessing bootstrap init" in out
         assert "Exception: Crashing because called by _bootstrap" in out
 
-    def test_bug890(self):
+    def test_bug_890(self):
         # chdir in multiprocessing shouldn't keep us from finding the
         # .coveragerc file.
         self.make_file("multi.py", """\
@@ -584,3 +693,83 @@ def test_thread_safe_save_data(tmpdir):
     finally:
         os.chdir(old_dir)
         should_run[0] = False
+
+
+@pytest.mark.skipif(env.WINDOWS, reason="SIGTERM doesn't work the same on Windows")
+@flaky(max_runs=3)          # Sometimes a test fails due to inherent randomness. Try more times.
+class SigtermTest(CoverageTest):
+    """Tests of our handling of SIGTERM."""
+
+    def test_sigterm_saves_data(self):
+        # A terminated process should save its coverage data.
+        self.make_file("clobbered.py", """\
+            import multiprocessing
+            import time
+
+            def subproc(x):
+                if x.value == 3:
+                    print("THREE", flush=True)  # line 6, missed
+                else:
+                    print("NOT THREE", flush=True)
+                x.value = 0
+                time.sleep(60)
+
+            if __name__ == "__main__":
+                print("START", flush=True)
+                x = multiprocessing.Value("L", 1)
+                proc = multiprocessing.Process(target=subproc, args=(x,))
+                proc.start()
+                while x.value != 0:
+                    time.sleep(.05)
+                proc.terminate()
+                print("END", flush=True)
+            """)
+        self.make_file(".coveragerc", """\
+            [run]
+            parallel = True
+            concurrency = multiprocessing
+            """)
+        out = self.run_command("coverage run clobbered.py")
+        # Under the Python tracer on Linux, we get the "Trace function changed"
+        # message. Does that matter?
+        if "Trace function changed" in out:
+            lines = out.splitlines(True)
+            assert len(lines) == 5  # "trace function changed" and "self.warn("
+            out = "".join(lines[:3])
+        assert out == "START\nNOT THREE\nEND\n"
+        self.run_command("coverage combine")
+        out = self.run_command("coverage report -m")
+        assert self.squeezed_lines(out)[2] == "clobbered.py 17 1 94% 6"
+
+    def test_sigterm_still_runs(self):
+        # A terminated process still runs its own SIGTERM handler.
+        self.make_file("handler.py", """\
+            import multiprocessing
+            import signal
+            import time
+
+            def subproc(x):
+                print("START", flush=True)
+                def on_sigterm(signum, frame):
+                    print("SIGTERM", flush=True)
+
+                signal.signal(signal.SIGTERM, on_sigterm)
+                x.value = 0
+                time.sleep(.1)
+                print("END", flush=True)
+
+            if __name__ == "__main__":
+                x = multiprocessing.Value("L", 1)
+                proc = multiprocessing.Process(target=subproc, args=(x,))
+                proc.start()
+                while x.value != 0:
+                    time.sleep(.02)
+                proc.terminate()
+            """)
+        self.make_file(".coveragerc", """\
+            [run]
+            parallel = True
+            concurrency = multiprocessing
+            """)
+        out = self.run_command("coverage run handler.py")
+        assert out == "START\nSIGTERM\nEND\n"
