@@ -16,7 +16,7 @@ import pytest
 from coverage.data import CoverageData, combine_parallel_data
 from coverage.data import add_data_to_hash, line_counts
 from coverage.debug import DebugControlString
-from coverage.exceptions import CoverageException
+from coverage.exceptions import DataError, NoDataError
 from coverage.files import PathAliases, canonical_filename
 
 from tests.coveragetest import CoverageTest
@@ -164,14 +164,14 @@ class CoverageDataTest(CoverageTest):
         covdata = DebugCoverageData()
         covdata.add_lines(LINES_1)
         msg = "Can't add branch measurements to existing line data"
-        with pytest.raises(CoverageException, match=msg):
+        with pytest.raises(DataError, match=msg):
             covdata.add_arcs(ARCS_3)
 
     def test_cant_add_lines_with_arcs(self):
         covdata = DebugCoverageData()
         covdata.add_arcs(ARCS_3)
         msg = "Can't add line measurements to existing branch data"
-        with pytest.raises(CoverageException, match=msg):
+        with pytest.raises(DataError, match=msg):
             covdata.add_lines(LINES_1)
 
     def test_touch_file_with_lines(self):
@@ -290,9 +290,9 @@ class CoverageDataTest(CoverageTest):
     def test_file_tracer_name(self):
         covdata = DebugCoverageData()
         covdata.add_lines({
-            "p1.foo": dict.fromkeys([1, 2, 3]),
-            "p2.html": dict.fromkeys([10, 11, 12]),
-            "main.py": dict.fromkeys([20]),
+            "p1.foo": [1, 2, 3],
+            "p2.html": [10, 11, 12],
+            "main.py": [20],
         })
         covdata.add_file_tracers({"p1.foo": "p1.plugin", "p2.html": "p2.plugin"})
         assert covdata.file_tracer("p1.foo") == "p1.plugin"
@@ -303,8 +303,8 @@ class CoverageDataTest(CoverageTest):
     def test_ok_to_repeat_file_tracer(self):
         covdata = DebugCoverageData()
         covdata.add_lines({
-            "p1.foo": dict.fromkeys([1, 2, 3]),
-            "p2.html": dict.fromkeys([10, 11, 12]),
+            "p1.foo": [1, 2, 3],
+            "p2.html": [10, 11, 12],
         })
         covdata.add_file_tracers({"p1.foo": "p1.plugin", "p2.html": "p2.plugin"})
         covdata.add_file_tracers({"p1.foo": "p1.plugin"})
@@ -313,9 +313,9 @@ class CoverageDataTest(CoverageTest):
     def test_ok_to_set_empty_file_tracer(self):
         covdata = DebugCoverageData()
         covdata.add_lines({
-            "p1.foo": dict.fromkeys([1, 2, 3]),
-            "p2.html": dict.fromkeys([10, 11, 12]),
-            "main.py": dict.fromkeys([20]),
+            "p1.foo": [1, 2, 3],
+            "p2.html": [10, 11, 12],
+            "main.py": [20],
         })
         covdata.add_file_tracers({"p1.foo": "p1.plugin", "main.py": ""})
         assert covdata.file_tracer("p1.foo") == "p1.plugin"
@@ -324,20 +324,20 @@ class CoverageDataTest(CoverageTest):
     def test_cant_file_tracer_unmeasured_files(self):
         covdata = DebugCoverageData()
         msg = "Can't add file tracer data for unmeasured file 'p1.foo'"
-        with pytest.raises(CoverageException, match=msg):
+        with pytest.raises(DataError, match=msg):
             covdata.add_file_tracers({"p1.foo": "p1.plugin"})
 
-        covdata.add_lines({"p2.html": dict.fromkeys([10, 11, 12])})
-        with pytest.raises(CoverageException, match=msg):
+        covdata.add_lines({"p2.html": [10, 11, 12]})
+        with pytest.raises(DataError, match=msg):
             covdata.add_file_tracers({"p1.foo": "p1.plugin"})
 
     def test_cant_change_file_tracer_name(self):
         covdata = DebugCoverageData()
-        covdata.add_lines({"p1.foo": dict.fromkeys([1, 2, 3])})
+        covdata.add_lines({"p1.foo": [1, 2, 3]})
         covdata.add_file_tracers({"p1.foo": "p1.plugin"})
 
         msg = "Conflicting file tracer name for 'p1.foo': 'p1.plugin' vs 'p1.plugin.foo'"
-        with pytest.raises(CoverageException, match=msg):
+        with pytest.raises(DataError, match=msg):
             covdata.add_file_tracers({"p1.foo": "p1.plugin.foo"})
 
     def test_update_lines(self):
@@ -375,18 +375,18 @@ class CoverageDataTest(CoverageTest):
         covdata2 = DebugCoverageData(suffix='2')
         covdata2.add_arcs(ARCS_3)
 
-        with pytest.raises(CoverageException, match="Can't combine arc data with line data"):
+        with pytest.raises(DataError, match="Can't combine arc data with line data"):
             covdata1.update(covdata2)
 
-        with pytest.raises(CoverageException, match="Can't combine line data with arc data"):
+        with pytest.raises(DataError, match="Can't combine line data with arc data"):
             covdata2.update(covdata1)
 
     def test_update_file_tracers(self):
         covdata1 = DebugCoverageData(suffix='1')
         covdata1.add_lines({
-            "p1.html": dict.fromkeys([1, 2, 3, 4]),
-            "p2.html": dict.fromkeys([5, 6, 7]),
-            "main.py": dict.fromkeys([10, 11, 12]),
+            "p1.html": [1, 2, 3, 4],
+            "p2.html": [5, 6, 7],
+            "main.py": [10, 11, 12],
         })
         covdata1.add_file_tracers({
             "p1.html": "html.plugin",
@@ -395,10 +395,10 @@ class CoverageDataTest(CoverageTest):
 
         covdata2 = DebugCoverageData(suffix='2')
         covdata2.add_lines({
-            "p1.html": dict.fromkeys([3, 4, 5, 6]),
-            "p2.html": dict.fromkeys([7, 8, 9]),
-            "p3.foo": dict.fromkeys([1000, 1001]),
-            "main.py": dict.fromkeys([10, 11, 12]),
+            "p1.html": [3, 4, 5, 6],
+            "p2.html": [7, 8, 9],
+            "p3.foo": [1000, 1001],
+            "main.py": [10, 11, 12],
         })
         covdata2.add_file_tracers({
             "p1.html": "html.plugin",
@@ -416,35 +416,35 @@ class CoverageDataTest(CoverageTest):
 
     def test_update_conflicting_file_tracers(self):
         covdata1 = DebugCoverageData(suffix='1')
-        covdata1.add_lines({"p1.html": dict.fromkeys([1, 2, 3])})
+        covdata1.add_lines({"p1.html": [1, 2, 3]})
         covdata1.add_file_tracers({"p1.html": "html.plugin"})
 
         covdata2 = DebugCoverageData(suffix='2')
-        covdata2.add_lines({"p1.html": dict.fromkeys([1, 2, 3])})
+        covdata2.add_lines({"p1.html": [1, 2, 3]})
         covdata2.add_file_tracers({"p1.html": "html.other_plugin"})
 
         msg = "Conflicting file tracer name for 'p1.html': 'html.plugin' vs 'html.other_plugin'"
-        with pytest.raises(CoverageException, match=msg):
+        with pytest.raises(DataError, match=msg):
             covdata1.update(covdata2)
 
         msg = "Conflicting file tracer name for 'p1.html': 'html.other_plugin' vs 'html.plugin'"
-        with pytest.raises(CoverageException, match=msg):
+        with pytest.raises(DataError, match=msg):
             covdata2.update(covdata1)
 
     def test_update_file_tracer_vs_no_file_tracer(self):
         covdata1 = DebugCoverageData(suffix="1")
-        covdata1.add_lines({"p1.html": dict.fromkeys([1, 2, 3])})
+        covdata1.add_lines({"p1.html": [1, 2, 3]})
         covdata1.add_file_tracers({"p1.html": "html.plugin"})
 
         covdata2 = DebugCoverageData(suffix="2")
-        covdata2.add_lines({"p1.html": dict.fromkeys([1, 2, 3])})
+        covdata2.add_lines({"p1.html": [1, 2, 3]})
 
         msg = "Conflicting file tracer name for 'p1.html': 'html.plugin' vs ''"
-        with pytest.raises(CoverageException, match=msg):
+        with pytest.raises(DataError, match=msg):
             covdata1.update(covdata2)
 
         msg = "Conflicting file tracer name for 'p1.html': '' vs 'html.plugin'"
-        with pytest.raises(CoverageException, match=msg):
+        with pytest.raises(DataError, match=msg):
             covdata2.update(covdata1)
 
     def test_update_lines_empty(self):
@@ -529,7 +529,7 @@ class CoverageDataTest(CoverageTest):
     def test_cant_touch_in_empty_data(self):
         covdata = DebugCoverageData()
         msg = "Can't touch files in an empty CoverageData"
-        with pytest.raises(CoverageException, match=msg):
+        with pytest.raises(DataError, match=msg):
             covdata.touch_file("abc.py")
 
     def test_read_and_write_are_opposites(self):
@@ -559,7 +559,7 @@ class CoverageDataTest(CoverageTest):
             t.join()
 
         assert_lines1_data(covdata)
-        assert exceptions == []
+        assert not exceptions
 
 
 class CoverageDataInTempDirTest(CoverageTest):
@@ -584,31 +584,23 @@ class CoverageDataInTempDirTest(CoverageTest):
         assert_arcs3_data(covdata2)
 
     def test_read_errors(self):
-        msg = r"Couldn't .* '.*[/\\]{}': \S+"
-
         self.make_file("xyzzy.dat", "xyzzy")
-        with pytest.raises(CoverageException, match=msg.format("xyzzy.dat")):
+        with pytest.raises(DataError, match=r"Couldn't .* '.*[/\\]xyzzy.dat': \S+"):
             covdata = DebugCoverageData("xyzzy.dat")
             covdata.read()
         assert not covdata
 
-        self.make_file("empty.dat", "")
-        with pytest.raises(CoverageException, match=msg.format("empty.dat")):
-            covdata = DebugCoverageData("empty.dat")
-            covdata.read()
-        assert not covdata
-
     def test_hard_read_error(self):
         self.make_file("noperms.dat", "go away")
         os.chmod("noperms.dat", 0)
-        with pytest.raises(CoverageException, match=r"Couldn't .* '.*[/\\]noperms.dat': "):
+        with pytest.raises(DataError, match=r"Couldn't .* '.*[/\\]noperms.dat': \S+"):
             covdata = DebugCoverageData("noperms.dat")
             covdata.read()
 
     @pytest.mark.parametrize("klass", [CoverageData, DebugCoverageData])
     def test_error_when_closing(self, klass):
         msg = r"Couldn't .* '.*[/\\]flaked.dat': \S+"
-        with pytest.raises(CoverageException, match=msg):
+        with pytest.raises(DataError, match=msg):
             covdata = klass("flaked.dat")
             covdata.add_lines(LINES_1)
             # I don't know how to make a real error, so let's fake one.
@@ -616,21 +608,23 @@ class CoverageDataInTempDirTest(CoverageTest):
             sqldb.close = lambda: 1/0
             covdata.add_lines(LINES_1)
 
-    def test_read_sql_errors(self):
+    def test_wrong_schema_version(self):
         with sqlite3.connect("wrong_schema.db") as con:
             con.execute("create table coverage_schema (version integer)")
             con.execute("insert into coverage_schema (version) values (99)")
         msg = r"Couldn't .* '.*[/\\]wrong_schema.db': wrong schema: 99 instead of \d+"
-        with pytest.raises(CoverageException, match=msg):
+        with pytest.raises(DataError, match=msg):
             covdata = DebugCoverageData("wrong_schema.db")
             covdata.read()
         assert not covdata
 
-        with sqlite3.connect("no_schema.db") as con:
-            con.execute("create table foobar (baz text)")
-        msg = r"Couldn't .* '.*[/\\]no_schema.db': \S+"
-        with pytest.raises(CoverageException, match=msg):
-            covdata = DebugCoverageData("no_schema.db")
+    def test_wrong_schema_schema(self):
+        with sqlite3.connect("wrong_schema_schema.db") as con:
+            con.execute("create table coverage_schema (xyzzy integer)")
+            con.execute("insert into coverage_schema (xyzzy) values (99)")
+        msg = r"Data file .* doesn't seem to be a coverage data file: .* no such column"
+        with pytest.raises(DataError, match=msg):
+            covdata = DebugCoverageData("wrong_schema_schema.db")
             covdata.read()
         assert not covdata
 
@@ -667,7 +661,8 @@ class CoverageDataFilesTest(CoverageTest):
 
         assert re.search(
             r"^Erasing data file '.*\.coverage'\n" +
-            r"Creating data file '.*\.coverage'\n" +
+            r"Opening data file '.*\.coverage'\n" +
+            r"Initing data file '.*\.coverage'\n" +
             r"Opening data file '.*\.coverage'\n$",
             debug.get_output()
         )
@@ -761,9 +756,9 @@ class CoverageDataFilesTest(CoverageTest):
     def test_combining_with_aliases(self):
         covdata1 = DebugCoverageData(suffix='1')
         covdata1.add_lines({
-            '/home/ned/proj/src/a.py': {1: None, 2: None},
-            '/home/ned/proj/src/sub/b.py': {3: None},
-            '/home/ned/proj/src/template.html': {10: None},
+            '/home/ned/proj/src/a.py': {1, 2},
+            '/home/ned/proj/src/sub/b.py': {3},
+            '/home/ned/proj/src/template.html': {10},
         })
         covdata1.add_file_tracers({
             '/home/ned/proj/src/template.html': 'html.plugin',
@@ -772,8 +767,8 @@ class CoverageDataFilesTest(CoverageTest):
 
         covdata2 = DebugCoverageData(suffix='2')
         covdata2.add_lines({
-            r'c:\ned\test\a.py': {4: None, 5: None},
-            r'c:\ned\test\sub\b.py': {3: None, 6: None},
+            r'c:\ned\test\a.py': {4, 5},
+            r'c:\ned\test\sub\b.py': {3, 6},
         })
         covdata2.write()
 
@@ -854,7 +849,7 @@ class CoverageDataFilesTest(CoverageTest):
     def test_combining_from_nonexistent_directories(self):
         covdata = DebugCoverageData()
         msg = "Couldn't combine from non-existent path 'xyzzy'"
-        with pytest.raises(CoverageException, match=msg):
+        with pytest.raises(NoDataError, match=msg):
             combine_parallel_data(covdata, data_paths=['xyzzy'])
 
     def test_interleaved_erasing_bug716(self):
@@ -894,5 +889,5 @@ class DumpsLoadsTest(CoverageTest):
             re.escape(repr(bad_data[:40])),
             len(bad_data),
             )
-        with pytest.raises(CoverageException, match=msg):
+        with pytest.raises(DataError, match=msg):
             covdata.loads(bad_data)
