@@ -134,7 +134,7 @@ class RecursionTest(CoverageTest):
         if pytrace:                                 # pragma: no metacov
             assert cov._warnings == ["Trace function changed, measurement is likely wrong: None"]
         else:
-            assert cov._warnings == []
+            assert not cov._warnings
 
 
 class MemoryLeakTest(CoverageTest):
@@ -475,7 +475,7 @@ class GettraceTest(CoverageTest):
 
             def tracer(frame, event, arg):
                 filename = os.path.basename(frame.f_code.co_filename)
-                print("%s: %s @ %d" % (event, filename, frame.f_lineno))
+                print(f"{event}: {filename} @ {frame.f_lineno}")
                 return tracer
 
             def begin():
