@@ -5,6 +5,7 @@
 
 import inspect
 import io
+import math
 import os.path
 from xml.etree import ElementTree
 
@@ -14,7 +15,7 @@ import coverage
 from coverage import env
 from coverage.control import Plugins
 from coverage.data import line_counts
-from coverage.exceptions import CoverageException, CoverageWarning
+from coverage.exceptions import CoverageWarning, NoSource, PluginError
 from coverage.misc import import_local_file
 
 import coverage.plugin
@@ -133,7 +134,7 @@ class LoadPluginsTest(CoverageTest):
             Nothing = 0
             """)
         msg_pat = "Plugin module 'no_plugin' didn't define a coverage_init function"
-        with pytest.raises(CoverageException, match=msg_pat):
+        with pytest.raises(PluginError, match=msg_pat):
             list(Plugins.load_plugins(["no_plugin"], None))
 
 
@@ -426,7 +427,7 @@ class GoodFileTracerTest(FileTracerTest):
             'TOTAL           11      7      0      0    36%',
             ]
         assert expected == report
-        assert round(abs(total-36.36), 2) == 0
+        assert math.isclose(total, 4 / 11 * 100)
 
     def test_plugin2_with_html_report(self):
         self.make_render_and_caller()
@@ -437,7 +438,7 @@ class GoodFileTracerTest(FileTracerTest):
         self.start_import_stop(cov, "caller")
 
         total = cov.html_report(include=["*.html"], omit=["uni*.html"])
-        assert round(abs(total-36.36), 2) == 0
+        assert math.isclose(total, 4 / 11 * 100)
 
         self.assert_exists("htmlcov/index.html")
         self.assert_exists("htmlcov/bar_4_html.html")
@@ -452,7 +453,7 @@ class GoodFileTracerTest(FileTracerTest):
         self.start_import_stop(cov, "caller")
 
         total = cov.xml_report(include=["*.html"], omit=["uni*.html"])
-        assert round(abs(total-36.36), 2) == 0
+        assert math.isclose(total, 4 / 11 * 100)
 
         dom = ElementTree.parse("coverage.xml")
         classes = {}
@@ -576,7 +577,7 @@ class GoodFileTracerTest(FileTracerTest):
 
         # But completely new filenames are not in the results.
         assert len(cov.get_data().measured_files()) == 3
-        with pytest.raises(CoverageException):
+        with pytest.raises(NoSource):
             cov.analysis("fictional.py")
 
 
@@ -988,7 +989,7 @@ class DynamicContextPluginTest(CoverageTest):
         """Make some files to use while testing dynamic context plugins."""
         self.make_file("rendering.py", """\
             def html_tag(tag, content):
-                return '<%s>%s</%s>' % (tag, content, tag)
+                return f'<{tag}>{content}</{tag}>'
 
             def render_paragraph(text):
                 return html_tag('p', text)
